package com.reactnative.socialcommerce.auth

import android.app.Activity
import android.content.Intent
import androidx.appcompat.app.AppCompatActivity
import androidx.lifecycle.lifecycleScope
import com.facebook.react.bridge.*
import com.loopnow.library.auth.activity.LoginWebViewActivity
import com.loopnow.library.auth.api.Api
import com.loopnow.library.auth.util.Key
import com.loopnow.library.camera.framework.auth.model.Credential
import com.reactnative.socialcommerce.auth.model.*
import com.reactnative.socialcommerce.auth.util.FWDataConvertUtil
import com.reactnative.socialcommerce.model.weakProperty
import com.reactnative.socialcommerce.util.asMap
import kotlinx.coroutines.async
import kotlinx.coroutines.launch
import java.lang.Exception


class AuthenticationModule(reactContext: ReactApplicationContext) :
    ReactContextBaseJavaModule(reactContext), AuthenticationInterface {

    private var loginPromise by weakProperty<Promise?>()

    private val activityEventListener =
        object : BaseActivityEventListener() {
            override fun onActivityResult(
                activity: Activity?,
                requestCode: Int,
                resultCode: Int,
                intent: Intent?
            ) {
                if (requestCode == REQUEST_RESULT) {
                    loginPromise?.let { promise ->
                        when (resultCode) {
                            Activity.RESULT_CANCELED ->
                                promise.resolve(FWLoginResult(true, null).asMap())
                            Activity.RESULT_OK -> {
                                val credential = intent?.getSerializableExtra(Key.USER_LOGIN_CREDENTIAL) as Credential?
                                credential?.let {
                                    val loginResult =
                                        FWLoginResult(false, FWCredential(
                                            credential.accessToken,
                                            getAccessTokenExpiredTime()
                                        ))
                                    promise.resolve(loginResult.asMap())
                                }
                                    ?: promise.resolve(FWLoginResult(true, null).asMap())
                            }
                        }

                        loginPromise = null
                    }
                }
            }
        }

    init {
        reactContext.addActivityEventListener(activityEventListener)
    }

    @ReactMethod
    override fun login(promise: Promise) {
        val activity = currentActivity

        if (activity == null) {
            loginPromise?.resolve(FWLoginResult(true, null).asMap())
            return
        }

        loginPromise = promise

        try {
            val intent = Intent(activity, LoginWebViewActivity::class.java)
            intent.flags = Intent.FLAG_ACTIVITY_SINGLE_TOP
            activity.startActivityForResult(intent, REQUEST_RESULT)
        } catch (t: Throwable) {
            loginPromise?.resolve(FWLoginResult(true, null).asMap())
            loginPromise = null
        }
    }

    @ReactMethod
    override fun getAccessToken(promise: Promise) {
        val orgInfo = Api.currentOrgInfo
        val me = Api.me
        if (orgInfo == null || me == null) {
            promise.resolve(null)
            return
        }

        val accessToken = Api.credential?.accessToken

        if (accessToken.isNullOrBlank()) {
            promise.resolve(null)
            return
        }
        promise.resolve(
            FWCredential(
                accessToken,
                getAccessTokenExpiredTime()
            ).asMap()
        )
    }

    @ReactMethod
    override fun getMerchantToken(promise: Promise) {
        val credential = Api.credential

        if (credential?.accessToken == null) {
            promise.resolve(null)
            return
        }

        promise.resolve(
            FWCredential(
                credential.accessToken,
                getAccessTokenExpiredTime()
            ).asMap()
        )
    }

    @ReactMethod
    override fun logout() {
        Api.logout()
    }

    @ReactMethod
    override fun syncCurrentOrganization(promise: Promise) {
        val page = 1
        val pageSize = 30
        (currentActivity as AppCompatActivity).lifecycleScope.launch {
            try {
                val meDef = async {
                    Api.getMeAsync()
                }
                val businessListDef = async {
                    Api.getBusinessList(page, pageSize, true)
                }
                meDef.await()
                val businessList = businessListDef.await()
                if (businessList.isEmpty()) {
                    throw IllegalArgumentException("business is null")
                }
                val defaultBusiness = businessList[0]
                val businessId = defaultBusiness.id
                if (businessId.isNullOrBlank()) {
                    throw IllegalArgumentException("businessId is null")
                }

                val channelListDef = async {
                    Api.getChannelList(businessId,page, pageSize, true)
                }
                val channelList = channelListDef.await()
                if (channelList.isEmpty()) {
                    throw IllegalArgumentException("channel is null")
                }

                val defaultChannel = channelList[0]
                Api.updateCurrentOrgInfo(
                    defaultBusiness,
                    defaultChannel
                )

                promise.resolve(true)
            } catch (e: Exception) {
                promise.resolve(false)
            }
        }
    }

    @ReactMethod
    override fun getBusinessList(page: Int?, pageSize: Int?, promise: Promise) {
        val p = if (page == null || page == 0) 1 else page
        (currentActivity as AppCompatActivity).lifecycleScope.launch {
            try {
                val businessList = Api.getBusinessList(p, pageSize?:30)
                val fwBusinessResponseData = FWBusinessResponseData(
                    FWDataConvertUtil.convertBusinessList2RN(businessList),
                    page,
                    pageSize
                )
                promise.resolve(fwBusinessResponseData.asMap())
            } catch (e: Exception) {
                promise.resolve(null)
            }
        }
    }

    @ReactMethod
    override fun getChannelList(businessId: String?, page: Int?, pageSize: Int?, promise: Promise) {
        val p = if (page == null || page == 0) 1 else page
        (currentActivity as AppCompatActivity).lifecycleScope.launch {
            try {
                val channelList = Api.getChannelList(businessId?:"", p, pageSize?:30)
                promise.resolve(
                    FWChannelResponseData(
                        FWDataConvertUtil.convertChannelList2RN(channelList),
                        page,
                        pageSize
                    ).asMap()
                )
            } catch (e: Exception) {
                promise.resolve(null)
            }
        }
    }

    @ReactMethod
    override fun changeCurrentChannel(businessId: String?, channelId: String?, promise: Promise) {
        try {
            Api.changeCurrentChannel(businessId, channelId)
            promise.resolve(true)
        } catch (e: Exception) {
            promise.resolve(false)
        }
    }

    @ReactMethod
    override fun getCurrentOrgInfo(promise: Promise) {
        val orgInfo = Api.currentOrgInfo
        val me = Api.me
        if (orgInfo == null || me == null) {
            promise.resolve(null)
            return
        }
        val fwOrgInfo = FWOrgInfo(
            FWDataConvertUtil.convertBusiness2RN(orgInfo.business),
            FWDataConvertUtil.convertChannel2RN(orgInfo.channel),
            FWDataConvertUtil.convertMe2RN(me),
        )
        promise.resolve(fwOrgInfo.asMap())
    }

    @ReactMethod
    fun addListener(eventName: String?, promise: Promise) {
        promise.resolve(Arguments.createMap())
    }

    @ReactMethod
    fun removeListeners(count: Int?, promise: Promise) {
        promise.resolve(Arguments.createMap())
    }

    override fun getName(): String {
        return "AuthenticationModule"
    }

    /**
     * unit: second
     */
    private fun getAccessTokenExpiredTime(): Long {
        return Api.getAccessTokenExpiredTime() / 1000
    }

    companion object {
        private const val REQUEST_RESULT = 1001
    }
}
