//
//  AuthenticationModule.swift
//  react-native-social-commerce
//
//  Created by Jeff Zheng on 2022/4/14.
//

import Foundation
import Combine
import React
import UserCenter
import RestClient
import Authentication

@objc(AuthenticationModule)
class AuthenticationModule: NSObject {
    var loginResolver: RCTPromiseResolveBlock?
    var loginRejecter: RCTPromiseRejectBlock?
    var refreshResolver: RCTPromiseResolveBlock?
    var refreshRejecter: RCTPromiseRejectBlock?

    var cancellabl: AnyCancellable?

    override init() {
        super.init()

        cancellabl = UserCenterManager.shared.credentialObserveable.sink { _ in

        } receiveValue: { [weak self] (result: Result<RefreshableCredential, AuthenticationError>) in
            guard let self = self else {
                return
            }

            switch result {
            case .success(let credential):
                let loginResult: LoginResult = self.convertCredential(refreshCredential: credential)
                let jsonObject = JsonHelper.fromObject(object: loginResult)
                if let loginResolver = self.loginResolver {
                    loginResolver(jsonObject)
                }
                if let refreshResolver = self.refreshResolver {
                    refreshResolver(jsonObject)
                }
            case .failure(let error):
                let fwError = self.convertError(error: error)
                if let loginRejecter = self.loginRejecter {
                    loginRejecter(fwError.name, fwError.reason, nil)
                }
                if let refreshRejecter = self.refreshRejecter {
                    refreshRejecter(fwError.name, fwError.reason, nil)
                }

                print("login error = \(error.localizedDescription)")
            }

            self.loginResolver = nil
            self.loginRejecter = nil
            self.refreshRejecter = nil
            self.refreshResolver = nil
        }
    }

    deinit {
        cancellabl?.cancel()
    }

    @objc static func requiresMainQueueSetup() -> Bool {
        return false
    }

    @objc(loginResolver:rejecter:)
    func login(resolver: @escaping RCTPromiseResolveBlock, rejecter: @escaping RCTPromiseRejectBlock) {
        DispatchQueue.main.async { [weak self] in
            let viewController = UIViewController.presentedViewController()

            if let viewController = viewController {
                UserCenterManager.shared.login(viewController)
                self?.loginResolver = resolver
                self?.loginRejecter = rejecter
            }
        }
    }

    @objc(syncCurrentOrganizationResolver:rejecter:)
    func syncCurrentOrganization(resolver: @escaping RCTPromiseResolveBlock,
                                 rejecter: @escaping RCTPromiseRejectBlock) {
        DispatchQueue.global(qos: .default).async {
            UserCenterManager.shared.syncCurrentOrganizationInfo { result in
              switch result {
              case .success:
                  resolver(true)
              case .failure(let error):
                  let error = self.convertError(error: error)
                  rejecter(error.name, error.reason, error)
              }
            }
        }
    }

    @objc(getAccessTokenResolver:rejecter:)
    func getAccessToken(resolver: @escaping RCTPromiseResolveBlock, rejecter: @escaping RCTPromiseRejectBlock) {
        DispatchQueue.global(qos: .default).async {
            let credential = UserCenterManager.shared.getAccessToken()
            let checked = UserCenterManager.shared.checkCurrentOrganization()
            if let credential = credential, checked {
                let jsonObject: [String: Any]? = self.convertCredential(refreshCredential: credential)
                resolver(jsonObject)
            } else {
                resolver(nil)
            }
        }
    }

    @objc(getMerchantTokenResolver:rejecter:)
    func getMerchantToken(resolver: @escaping RCTPromiseResolveBlock, rejecter: @escaping RCTPromiseRejectBlock) {
        DispatchQueue.global(qos: .default).async {
            let result = UserCenterManager.shared.getMerchantToken()
            switch result {
            case .success(let credential):
                resolver(credential)
            case .failure(let error):
                let error = self.convertError(error: error)
                rejecter(error.name, error.reason, error)
            }
        }
    }

    @objc(refreshTokenResolver:rejecter:)
    func refreshToken(resolver: @escaping RCTPromiseResolveBlock, rejecter: @escaping RCTPromiseRejectBlock) {
        DispatchQueue.global(qos: .default).async {
            let canRefresh = UserCenterManager.shared.refreshAccessToken()
            if !canRefresh {
                // if can't refresh that indicates there's no need to refresh, because the token hasn't expired.
                let credential = UserCenterManager.shared.getAccessToken()
                if let credential = credential {
                    let jsonObject: [String: Any]? = self.convertCredential(refreshCredential: credential)
                    resolver(jsonObject)
                } else {
                    rejecter("unknow", "unknow", nil)
                }
            } else {
                self.refreshRejecter = rejecter
                self.refreshResolver = resolver
            }
        }
    }

    @objc(logout)
    func logout() {
        UserCenterManager.shared.logout()
    }
}
