import type BusinessResponseData from './models/Business';
import type ChannelResponseData from './models/Channel';
import type Credential from './models/Credential';
import type LoginResult from './models/LoginResult';
import type StoreResponseData from './models/Store';
import AuthenticationModule from './modules/AuthenticationModule';
import type CurrentOrganizationInfo from './models/CurrentOrganizationInfo';

class AuthenticationManager {
  private static instance?: AuthenticationManager;

  public static getInstance() {
    if (!AuthenticationManager.instance) {
      AuthenticationManager.instance = new AuthenticationManager();
    }

    return AuthenticationManager.instance;
  }

  /**
   * Host app can call this method to present a firework login screen, then can get an access token after the user authorizes it.
   */
  public login(): Promise<LoginResult> {
    return AuthenticationModule.login();
  }

  /**
   * After login success, the host needs to call this method to prepare firework’s basic info so that it can use firework’s feature.
   */
  public syncCurrentOrganization(): Promise<boolean> {
    return AuthenticationModule.syncCurrentOrganization();
  }

  /**
   * Host can call this method to get a token, if there is no token, the host needs to call the login method.
   */
  public getMerchantToken(): Promise<Object> {
    return AuthenticationModule.getMerchantToken();
  }

  public getAccessToken(): Promise<Credential> {
    return AuthenticationModule.getAccessToken();
  }

  /**
   * Host can call this method to refresh the token when the merchant token expires.
   */
  public refreshToken(): Promise<Credential> {
    return AuthenticationModule.refreshToken();
  }

  /**
   * Host needs to call this method to clear local token info when user tap logout
   */
  public logout(): void {
    AuthenticationModule.logout();
  }

  getBusinessList(
    page?: number,
    pageSize?: number,
  ): Promise<BusinessResponseData> {
    return AuthenticationModule.getBusinessList(page ?? 0, pageSize ?? 30);
  }

  getChannelList(
    businessId: string,
    page?: number,
    pageSize?: number,
  ): Promise<ChannelResponseData> {
    return AuthenticationModule.getChannelList(
      businessId,
      page ?? 0,
      pageSize ?? 30,
    );
  }

  getStoreList(
    businessId: string,
    page?: number,
    pageSize?: number,
  ): Promise<StoreResponseData> {
    return AuthenticationModule.getStoreList(
      businessId,
      page ?? 0,
      pageSize ?? 30,
    );
  }

  changeCurrentChannel(
    businessId: string,
    channelId: string,
  ): Promise<boolean> {
    return AuthenticationModule.changeCurrentChannel(businessId, channelId);
  }

  changeCurrentStore(businessId: string, storeId: string): Promise<boolean> {
    return AuthenticationModule.changeCurrentStore(businessId, storeId);
  }

  getCurrentOrgInfo(): Promise<CurrentOrganizationInfo> {
    return AuthenticationModule.getCurrentOrgInfo();
  }
}

export default AuthenticationManager;
